searchData={"items":[{"type":"module","title":"AshRateLimiter","doc":"An extension for `Ash.Resource` which adds the ability to rate limit access to actions.","ref":"AshRateLimiter.html"},{"type":"function","title":"AshRateLimiter.key_for_action/3","doc":"The default bucket key generation function for `AshRateLimiter`.\n\nGenerates a key based on the domain, resource and action names. For update and destroy actions it will also include the primary key(s) in the key.","ref":"AshRateLimiter.html#key_for_action/3"},{"type":"function","title":"Options - AshRateLimiter.key_for_action/3","doc":"* `:include_pk?` (`t:boolean/0`) - Whether or not to include the primary keys in the generated key in update/destroy actions. The default value is `true`.\n\n* `:include_actor_attributes` (one or a list of `t:atom/0`) - A list of attributes to include from the current actor The default value is `[]`.\n\n* `:include_tenant?` (`t:boolean/0`) - Whether or not to include the tenant in the bucket key. The default value is `false`.","ref":"AshRateLimiter.html#key_for_action/3-options"},{"type":"function","title":"Examples - AshRateLimiter.key_for_action/3","doc":"iex> Example.Post\n    ...> |> Ash.Changeset.for_create(:create, post_attrs())\n    ...> |> key_for_action(%{})\n    \"example/post/create\"\n\n    iex> Example.Post\n    ...> |> Ash.Query.for_read(:read)\n    ...> |> key_for_action(%{})\n    \"example/post/read\"\n\n    iex> generate_post(id: \"0196ebc0-83b4-7938-bc9d-22ae65dbec0b\")\n    ...> |> Ash.Changeset.for_update(:update, %{title: \"Local Farmer Claims 'Space Zombie' Wrecked His Barn\"})\n    ...> |> key_for_action(%{})\n    \"example/post/update?id=0196ebc0-83b4-7938-bc9d-22ae65dbec0b\"\n\n    iex> generate_post(id: \"0196ebc6-4839-7a9e-b2a6-feb0eabc8772\")\n    ...> |> Ash.Changeset.for_destroy(:destroy)\n    ...> |> key_for_action(%{})\n    \"example/post/destroy?id=0196ebc6-4839-7a9e-b2a6-feb0eabc8772\"\n\n    iex> generate_post(id: \"0196ebcd-fabf-74c3-9585-8b13c5f07068\")\n    ...> |> Ash.Changeset.for_destroy(:destroy)\n    ...> |> key_for_action(%{}, include_pk?: false)\n    \"example/post/destroy\"\n\n    iex> Example.Post\n    ...> |> Ash.Changeset.for_create(:create, post_attrs())\n    ...> |> key_for_action(%{actor: %{role: :time_traveller}}, include_actor_attributes: [:role])\n    \"example/post/create?actor%5Brole%5D=time_traveller\"\n\n    iex> Example.Post\n    ...> |> Ash.Changeset.new()\n    ...> |> Ash.Changeset.set_tenant(\"Hill Valley Telegraph\")\n    ...> |> Ash.Changeset.for_create(:create, post_attrs())\n    ...> |> key_for_action(%{}, include_tenant?: true)\n    \"example/post/create?tenant=Hill%20Valley%20Telegraph\"","ref":"AshRateLimiter.html#key_for_action/3-examples"},{"type":"type","title":"AshRateLimiter.keyfun/0","doc":"","ref":"AshRateLimiter.html#t:keyfun/0"},{"type":"macro","title":"AshRateLimiter.rate_limit/1","doc":"","ref":"AshRateLimiter.html#rate_limit/1"},{"type":"type","title":"AshRateLimiter.t/0","doc":"","ref":"AshRateLimiter.html#t:t/0"},{"type":"module","title":"AshRateLimiter.BuiltinChanges","doc":"Change helpers to allow direct usage in mutation actions.","ref":"AshRateLimiter.BuiltinChanges.html"},{"type":"function","title":"AshRateLimiter.BuiltinChanges.rate_limit/1","doc":"Add a rate-limit change directly to an action.","ref":"AshRateLimiter.BuiltinChanges.html#rate_limit/1"},{"type":"function","title":"Options - AshRateLimiter.BuiltinChanges.rate_limit/1","doc":"* `:limit` (`t:pos_integer/0`) - Required. The maximum number of events allowed within the given scale\n\n* `:per` (`t:pos_integer/0`) - Required. The time period (in milliseconds) for in which events are counted\n\n* `:key` - The key used to identify the event. See the docs for `AshRateLimiter.key_for_action/2` for more. The default value is `&AshRateLimiter.key_for_action/2`.","ref":"AshRateLimiter.BuiltinChanges.html#rate_limit/1-options"},{"type":"type","title":"AshRateLimiter.BuiltinChanges.rate_limit_options/0","doc":"","ref":"AshRateLimiter.BuiltinChanges.html#t:rate_limit_options/0"},{"type":"module","title":"AshRateLimiter.BuiltinPreparations","doc":"Preparation helpers to allow direct usage in read actions.","ref":"AshRateLimiter.BuiltinPreparations.html"},{"type":"function","title":"AshRateLimiter.BuiltinPreparations.rate_limit/1","doc":"Add a rate-limit change directly to an action.","ref":"AshRateLimiter.BuiltinPreparations.html#rate_limit/1"},{"type":"function","title":"Options - AshRateLimiter.BuiltinPreparations.rate_limit/1","doc":"* `:limit` (`t:pos_integer/0`) - Required. The maximum number of events allowed within the given scale\n\n* `:per` (`t:pos_integer/0`) - Required. The time period (in milliseconds) for in which events are counted\n\n* `:key` - The key used to identify the event. See the docs for `AshRateLimiter.key_for_action/2` for more. The default value is `&AshRateLimiter.key_for_action/2`.","ref":"AshRateLimiter.BuiltinPreparations.html#rate_limit/1-options"},{"type":"type","title":"AshRateLimiter.BuiltinPreparations.rate_limit_options/0","doc":"","ref":"AshRateLimiter.BuiltinPreparations.html#t:rate_limit_options/0"},{"type":"module","title":"AshRateLimiter.Change","doc":"A resource change which implements rate limiting.","ref":"AshRateLimiter.Change.html"},{"type":"module","title":"AshRateLimiter.Info","doc":"Auto-generated introspection functions.","ref":"AshRateLimiter.Info.html"},{"type":"function","title":"AshRateLimiter.Info.rate_limit/1","doc":"rate_limit DSL entities","ref":"AshRateLimiter.Info.html#rate_limit/1"},{"type":"function","title":"AshRateLimiter.Info.rate_limit_hammer/1","doc":"The hammer module to use for rate limiting","ref":"AshRateLimiter.Info.html#rate_limit_hammer/1"},{"type":"function","title":"AshRateLimiter.Info.rate_limit_hammer!/1","doc":"The hammer module to use for rate limiting","ref":"AshRateLimiter.Info.html#rate_limit_hammer!/1"},{"type":"function","title":"AshRateLimiter.Info.rate_limit_options/1","doc":"rate_limit DSL options\n\nReturns a map containing the and any configured or default values.","ref":"AshRateLimiter.Info.html#rate_limit_options/1"},{"type":"module","title":"AshRateLimiter.Preparation","doc":"A resource preparation which implements rate limiting.","ref":"AshRateLimiter.Preparation.html"},{"type":"module","title":"AshRateLimiter.Transformer","doc":"A Spark DSL transformer for rate limiting.","ref":"AshRateLimiter.Transformer.html"},{"type":"function","title":"AshRateLimiter.Transformer.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"AshRateLimiter.Transformer.html#after_compile?/0"},{"type":"function","title":"AshRateLimiter.Transformer.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"AshRateLimiter.Transformer.html#before?/1"},{"type":"exception","title":"AshRateLimiter.LimitExceeded","doc":"An exception which is raised or returned when an action invocation exceeds the defined limits.","ref":"AshRateLimiter.LimitExceeded.html"},{"type":"function","title":"AshRateLimiter.LimitExceeded.exception/1","doc":"Create an `Elixir.AshRateLimiter.LimitExceeded` without raising it.","ref":"AshRateLimiter.LimitExceeded.html#exception/1"},{"type":"function","title":"Keys - AshRateLimiter.LimitExceeded.exception/1","doc":"- :action\n- :hammer\n- :limit\n- :per\n- :key","ref":"AshRateLimiter.LimitExceeded.html#exception/1-keys"},{"type":"extras","title":"README","doc":"![Logo](https://github.com/ash-project/ash/blob/main/logos/cropped-for-header-black-text.png?raw=true#gh-light-mode-only)\n![Logo](https://github.com/ash-project/ash/blob/main/logos/cropped-for-header-white-text.png?raw=true#gh-dark-mode-only)\n\n[![Ash CI](https://github.com/ash-project/ash_rate_limiter/actions/workflows/elixir.yml/badge.svg)](https://github.com/ash-project/ash_rate_limiter/actions/workflows/elixir.yml)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n[![Hex version badge](https://img.shields.io/hexpm/v/ash_rate_limiter.svg)](https://hex.pm/packages/ash_rate_limiter)\n[![Hexdocs badge](https://img.shields.io/badge/docs-hexdocs-purple)](https://hexdocs.pm/ash_rate_limiter)\n\n# AshRateLimiter\n\nWelcome! This is an extension for the [Ash framework](https://hexdocs.pm/ash)\nwhich protects [actions](https://hexdocs.pm/ash/actions.html) from abuse by enforcing rate limits.\n\nUses the excellent [hammer](https://hex.pm/packages/hammer) to provide rate limiting functionality.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"Add `ash_rate_limiter` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ash_rate_limiter, \"~> 0.1.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - README","doc":"1. **Configure Hammer**: First, you need to configure a Hammer backend. For development, you can use the ETS backend:\n\n```elixir\n# config/config.exs\nconfig :hammer,\n  backend: {Hammer.Backend.ETS, []}\n```\n\nFor production, consider using Redis:\n\n```elixir\n# config/config.exs  \nconfig :hammer,\n  backend: {Hammer.Backend.Redis, [\n    host: \"localhost\",\n    port: 6379\n  ]}\n```\n\n2. **Add to your resource**: Use the `rate_limit` DSL section in your Ash resource:\n\n```elixir\ndefmodule MyApp.Post do\n  use Ash.Resource,\n    domain: MyApp,\n    extensions: [AshRateLimiter]\n\n  rate_limit do\n    # Configure hammer backend\n    hammer Hammer\n    \n    # Limit create action to 10 requests per 5 minutes\n    action :create, limit: 10, per: :timer.minutes(5)\n    \n    # Limit read action to 100 requests per minute  \n    action :read, limit: 100, per: :timer.minutes(1)\n  end\n\n  # ... rest of your resource definition\nend\n```\n\n3. **That's it!** Your actions are now rate limited. When the limit is exceeded, an `AshRateLimiter.LimitExceeded` error will be raised.","ref":"readme.html#quick-start"},{"type":"extras","title":"Basic Usage - README","doc":"","ref":"readme.html#basic-usage"},{"type":"extras","title":"Simple Rate Limiting - README","doc":"```elixir\nrate_limit do\n  hammer Hammer\n  action :create, limit: 5, per: :timer.minutes(1)\nend\n```","ref":"readme.html#simple-rate-limiting"},{"type":"extras","title":"Per-User Rate Limiting - README","doc":"```elixir\nrate_limit do\n  hammer Hammer\n  action :create, \n    limit: 10, \n    per: :timer.minutes(5),\n    key: fn changeset, context ->\n      \"user:#{context.actor.id}:create\"\n    end\nend\n```","ref":"readme.html#per-user-rate-limiting"},{"type":"extras","title":"Multiple Actions - README","doc":"```elixir\nrate_limit do\n  hammer Hammer\n  action :create, limit: 10, per: :timer.minutes(5)\n  action :update, limit: 20, per: :timer.minutes(5) \n  action :read, limit: 100, per: :timer.minutes(1)\nend\n```","ref":"readme.html#multiple-actions"},{"type":"extras","title":"Advanced Usage - README","doc":"","ref":"readme.html#advanced-usage"},{"type":"extras","title":"Manual Integration - README","doc":"For more control, you can add rate limiting directly to specific actions:\n\n```elixir\ndefmodule MyApp.Post do\n  use Ash.Resource, domain: MyApp\n\n  actions do\n    create :create do\n      change {AshRateLimiter.Change, limit: 10, per: :timer.minutes(5)}\n    end\n    \n    read :read do\n      prepare {AshRateLimiter.Preparation, limit: 100, per: :timer.minutes(1)}\n    end\n  end\nend\n```\n\nOr use the built-in helpers:\n\n```elixir\nactions do\n  create :create do\n    change rate_limit(limit: 10, per: :timer.minutes(5))\n  end\nend\n```","ref":"readme.html#manual-integration"},{"type":"extras","title":"Custom Key Functions - README","doc":"The key function determines how requests are grouped for rate limiting:\n\n```elixir\n# Rate limit per IP address\nkey: fn _changeset, context ->\n  \"ip:#{context[:ip_address]}\"\nend\n\n# Rate limit per user and action\nkey: fn changeset, context ->\n  \"user:#{context.actor.id}:action:#{changeset.action.name}\"\nend\n\n# Use the built-in key function with options\nkey: {&AshRateLimiter.key_for_action/2, include_actor_attributes: [:role]}\n```","ref":"readme.html#custom-key-functions"},{"type":"extras","title":"Error Handling - README","doc":"When rate limits are exceeded, an `AshRateLimiter.LimitExceeded` exception is raised:\n\n```elixir\ncase MyApp.create_post(attrs) do\n  {:ok, post} -> \n    # Success\n    {:ok, post}\n    \n  {:error, %AshRateLimiter.LimitExceeded{} = error} ->\n    # Rate limit exceeded\n    {:error, \"Too many requests, please try again later\"}\n    \n  {:error, other_error} ->\n    # Handle other errors\n    {:error, other_error}\nend\n```\n\nIn web applications, the exception includes `Plug.Exception` behaviour for automatic HTTP 429 responses.","ref":"readme.html#error-handling"},{"type":"extras","title":"Testing - README","doc":"In test environments, you may want to disable rate limiting or use a test-friendly configuration:\n\n```elixir\n# config/test.exs\nconfig :hammer,\n  backend: {Hammer.Backend.ETS, []}\n\n# Or disable rate limiting entirely in tests by using a mock\n```","ref":"readme.html#testing"},{"type":"extras","title":"Reference - README","doc":"- [AshRateLimiter DSL](documentation/dsls/DSL-AshRateLimiter.md)\n- [Hammer Documentation](https://hexdocs.pm/hammer)","ref":"readme.html#reference"},{"type":"extras","title":"Change Log","doc":"# Change Log\n\nAll notable changes to this project will be documented in this file.\nSee [Conventional Commits](Https://conventionalcommits.org) for commit guidelines.\n\n<!-- changelog -->","ref":"changelog.html"},{"type":"extras","title":"v0.1.1 (2025-05-31) - Change Log","doc":"","ref":"changelog.html#v0-1-1-2025-05-31"},{"type":"extras","title":"v0.1.0 (2025-05-21) - Change Log","doc":"","ref":"changelog.html#v0-1-0-2025-05-21"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 James Harton\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"AshRateLimiter","doc":"<!--\nThis file was generated by Spark. Do not edit it by hand.\n-->\n# AshRateLimiter\n\nAn extension for `Ash.Resource` which adds the ability to rate limit access to actions.","ref":"dsl-ashratelimiter.html"},{"type":"extras","title":"rate_limit - AshRateLimiter","doc":"Configure rate limiting for actions.\n\n#### Hammer\n\nThis library uses the [hammer](https://hex.pm/packages/hammer) package to provide\nrate limiting features.  See [hammer's documentation](https://hexdocs.pm/hammer) for more information.\n\n#### Keys\n\nHammer uses a \"key\" to identify which bucket to allocate an event against.  You can use this to tune the rate limit for specific users or events.\n\nYou can provide either a statically configured string key, or a function of arity one or two, which when given a query/changeset and optional context object can generate a key.\n\nThe default is `AshRateLimiter.key_for_action/2`. See it's docs for more information.","ref":"dsl-ashratelimiter.html#rate_limit"},{"type":"extras","title":"Nested DSLs - AshRateLimiter","doc":"* [action](#rate_limit-action)","ref":"dsl-ashratelimiter.html#nested-dsls"},{"type":"extras","title":"Examples - AshRateLimiter","doc":"```\nrate_limit do\n  action :create, limit: 10, per: :timer.minutes(5)\nend\n\n```","ref":"dsl-ashratelimiter.html#examples"},{"type":"extras","title":"Options - AshRateLimiter","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`hammer`](#rate_limit-hammer){: #rate_limit-hammer .spark-required} | `module` |  | The hammer module to use for rate limiting |","ref":"dsl-ashratelimiter.html#options"},{"type":"extras","title":"rate_limit.action - AshRateLimiter","doc":"```elixir\naction action\n```\n\n\nConfigure rate limiting for a single action.\n\nIt does this by adding a global change or preparation to the resource with the provided configuration.  For more advanced configuration you can add the change/preparation directly to your action using `AshRateLimiter.BuiltinChanges.rate_limit/1` or `AshRateLimiter.BuiltinPreparations.rate_limit/1`.","ref":"dsl-ashratelimiter.html#rate_limit-action"},{"type":"extras","title":"Arguments - AshRateLimiter","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`action`](#rate_limit-action-action){: #rate_limit-action-action .spark-required} | `atom` |  | The name of the action to limit |","ref":"dsl-ashratelimiter.html#arguments"},{"type":"extras","title":"Options - AshRateLimiter","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`limit`](#rate_limit-action-limit){: #rate_limit-action-limit .spark-required} | `pos_integer` |  | The maximum number of events allowed within the given period |\n| [`per`](#rate_limit-action-per){: #rate_limit-action-per .spark-required} | `pos_integer` |  | The time period (in milliseconds) for in which events are counted |\n| [`key`](#rate_limit-action-key){: #rate_limit-action-key } | `String.t \\| (any -> any) \\| (any, any -> any)` | `&AshRateLimiter.key_for_action/2` | The key used to identify the event. See above. |\n| [`description`](#rate_limit-action-description){: #rate_limit-action-description } | `String.t` |  | A description of the rate limit |","ref":"dsl-ashratelimiter.html#options-1"},{"type":"extras","title":"Introspection - AshRateLimiter","doc":"Target: `AshRateLimiter`\n\n\n\n\n\n .spark-required::after { content: \"*\"; color: red !important; }","ref":"dsl-ashratelimiter.html#introspection"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}